//+------------------------------------------------------------------+
//|                                                  MACD - RSI.mq4  |
//|                      Copyright  2009, MetaQuotes Software Corp. |
//|                      http://www.metaquotes.net/                  |
//|                      made in Russia by lukas1                    |
//+------------------------------------------------------------------+
#property  copyright "Copyright  2009, MetaQuotes Software Corp."
#property  link      "lukas1@ngs.ru"
//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 3
#property  indicator_color1  Silver
#property  indicator_color2  Red
#property  indicator_color3 DodgerBlue
#property  indicator_width1  2
#property  indicator_level1  0.002
#property  indicator_level2  0.001
#property  indicator_level3  0.0
#property  indicator_level4  -0.001
#property  indicator_level5  -0.002
//---- indicator parameters
extern int FastEMA=12;
extern int SlowEMA=26;
extern int SignalSMA=9;
//---- input parameters
extern int RSIPeriod=14;
extern double caliber=0.5; //need for RSI caliber
//----
double Caliber;
//---- buffers
double RSIBuffer[];
double PosBuffer[];
double NegBuffer[];
//---- indicator buffers
double     MacdBuffer[];
double     SignalBuffer[];
//----
double del;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
    switch(Period())
      {
      case     1: Caliber=0.5*caliber; del=0.5;  break;
      case     5: Caliber=1*caliber;   del=1.0; break;
      case    15: Caliber=1*caliber;   del=1.0;break;
      case    30: Caliber=2*caliber;   del=2.0; break;
      case    60: Caliber=2*caliber;   del=2.0;break;
      case   240: Caliber=4*caliber;   del=4.0;break;
      case  1440: Caliber=12*caliber;  del=5.0; break;
      case 10080: Caliber=20*caliber;  del=8.0;break;
      case 43200: Caliber=50*caliber;  del=14.0; break;
      }
   IndicatorBuffers(5);
//---- drawing settings
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexDrawBegin(1,SignalSMA);
   IndicatorDigits(Digits+1);
//---- indicator buffers mapping
   SetIndexBuffer(0,MacdBuffer);
   SetIndexBuffer(1,SignalBuffer);
   SetIndexBuffer(2,RSIBuffer);
   SetIndexBuffer(3,PosBuffer);
   SetIndexBuffer(4,NegBuffer);
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("MACD-RSI("+FastEMA+","+SlowEMA+","+SignalSMA+"),("+RSIPeriod+")");
   SetIndexLabel(0,"MACD");
   SetIndexLabel(1,"Signal");
   SetIndexLabel(2,"RSI");
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int start()
  {
   int limit;
   int counted_bars=IndicatorCounted();
   double rel,negative,positive;
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
//---- macd counted in the 1-st buffer
   for(int i=0; i<limit; i++)
      MacdBuffer[i]=(iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i))/del;
//---- signal line counted in the 2-nd buffer
   for(i=0; i<limit; i++)
      SignalBuffer[i]=iMAOnArray(MacdBuffer,Bars,SignalSMA,0,MODE_EMA,i)/del;
//=============================================================== RSI =
   i=Bars-RSIPeriod-1;
   if(counted_bars>=RSIPeriod) i=Bars-counted_bars-1;
   while(i>=0)
     {
      double sumn=0.0,sump=0.0;
      if(i==Bars-RSIPeriod-1)
        {
         int k=Bars-2;
         //---- initial accumulation
         while(k>=i)
           {
            rel=MacdBuffer[k]-MacdBuffer[k+1];
            if(rel>0) sump+=rel;
            else      sumn-=rel;
            k--;
           }
         positive=sump/RSIPeriod;
         negative=sumn/RSIPeriod;
        }
      else
        {
         //---- smoothed moving average
         rel=MacdBuffer[i]-MacdBuffer[i+1];
         if(rel>0) sump=rel;
         else      sumn=-rel;
         positive=(PosBuffer[i+1]*(RSIPeriod-1)+sump)/RSIPeriod;
         negative=(NegBuffer[i+1]*(RSIPeriod-1)+sumn)/RSIPeriod;
        }
      PosBuffer[i]=positive;
      NegBuffer[i]=negative;
      if(negative==0.0) RSIBuffer[i]=0.0;
      else RSIBuffer[i]=Caliber*Point*(50.0-100.0/(1+positive/negative) )/del;
      i--;
     }
   return(0);
  }
//+------------------------------------------------------------------+